require 'signpost/verifier'

module SignPost
  module Rails
    module Authentication # :nodoc:
      attr_accessor :signpost_users
      class << self
        attr_accessor :fetch_method
      end

      def authenticate_signpost
        session[:user_id] = Signpost::Verifier.verify_action_dispatch_request(
          request,
          authorized_keys
        )
      rescue Signpost::Verifier::VerificationError => e
        ::Rails.logger.warn "Auth failure: #{e}"
        head :unauthorized
        return
      end

      def authorized_keys
        # If cached for less than 5 mins, then return the cache
        return self.signpost_users[:authorized_keys_path] if self.signpost_users && (Time.now - self.signpost_users[:cached_at]) < 5 * 60
        keys = SignPost::Rails::Authentication.fetch_method.call
        self.signpost_users = { cached_at: Time.now, authorized_keys_path: keys }
        keys
      end
    end
  end
end
